class BotConnection extends TcpLink
	config(BotAPI);


var string ReceivedData;
var array<string> ReceivedArgs;
var array<string> ReceivedVals;

//used to store info inside function where local wont work
//set property text dosent seem to work with a local
var actor tempActor;
var bool tempBool;


var BotServer Parent;
var RemoteBot theBot;

// set true for verbose debug logs
var config bool bDebug;


// set true for iterative mode
var config bool bIterative;


// delay between visionUpdates
var config float visionTime;
var config bool bVision;

var	int attrNum;

// delimeters for strings sent to clients. set to match those of the server
// as = identifies the start of an attribute
var string as;
// ae = attribute end
var string ae;
// ib = item break
var string ib;

//------------Message Types---------------------------
// Each message appears on a seperate line. The first item
//  of each message is the type identifier. It is a three
//  letter code identifing what the message contains.
// Used types are:
//  (Asynchronous messages)
//  FIN - game ended
//  FAL - bot just hit a dropoff
//  NFO - info about gametype. cue to register self
//  AIN - added inventory. Bot got new inventory item
//  VMS - recieved message from global chat channel
//  VMT - recieved message from team chat channel
//  VMG - recieved tokenized message
//  ZCF - foot changed zones
//  ZCH - head changed zones
//  ZCB - bot changed zones
//  CWP - changed weapon
//  WAL - collided with a wall
//  BMP - bumped another actor
//  HRP - hear pickup
//  HRN - hear noise
//  SEE - see player - !!!DEPRICATED
//  PRJ - incoming projectile
//  KIL - some other player died
//  DIE - bot died
//  DAM - took damage
//  HIT - hurt another player
//  PTH - a series of nodes in response to a getpath call
//  RCH - a bool result of a checkreach call
// (Synchronous messages)
//  PLR - see another player
//  INV - see an inventory item (or weapon)
//  NAV - see a navigation point
//  DOM - see a domination point
//  FLG - see a flag
//  MOV - see a mover
//  END - end of a vision batch
//  BEG - begin of a vision batch
//  SLF - random information about bots state
//  GAM - random information about game state
//--------------------------------------------------



function PostBeginPlay()
{
    Parent = BotServer(Owner);
	ib = Parent.ib;
	as = Parent.as;
	ae = Parent.ae;

    if(bDebug)
    	log("Spawned BotConnection");
}

//Socket established
event Accepted()
{
	local string gameInfoStr, levelName;
	local int i;

    if(bDebug)
    	log("Accepted BotConnection" @ self);

 	//no clean way to do this since gametypes are not derived from
 	//a common ancestor
	switch(Parent.gameClass)
	{
		/*case "BotTeamGamePlus":
			gameInfoStr = BotTeamGamePlus(Level.Game).GetGameInfo();
			break;*/
		case "BotDeathMatch":
			gameInfoStr = BotDeathMatch(Level.Game).GetGameInfo();
			break;
		/*case "BotDomination":
			gameInfoStr = BotDomination(Level.Game).GetGameInfo();
			break;
		case "BotCTF":
			gameInfoStr = BotCTF(Level.Game).GetGameInfo();
			break;*/
	}

	levelName = string(Level);
	i = InStr(Caps(levelName), ".LEVELINFO");
	if(i != -1)
	levelName = Left(levelName, i);

	SendLine("NFO" $ib$as$ "Gametype" $ib$ Parent.gameClass $ae$ib$as$
		"Level" $ib$ levelName $ae$ gameInfoStr);
	gotoState('monitoring','WaitingForInit');
}

//InitRecieved from client
event InitRecieved()
{
local string clientName;
local string teamString;
local int teamNum;
local vector startLocation;
local rotator startRotation;
local string className;

  clientName = GetArgVal("Name");
  teamString = GetArgVal("Team");
  ParseVector(startLocation,"Location");
  ParseRot(startRotation);
  className = GetArgVal("ClassName");

  if( teamString == "" )
  	teamNum = 255;
  else
  	teamNum = int(teamString);

 	//no clean way to do this since gametypes are not derived from
 	//a common ancestor that has AddRemoteBot

	if(bDebug)
		log("InitRecieved");

	//switch(Parent.gameClass)
	//{
	//	case "BotTeamGamePlus":
			/*theBot = BotTeamGamePlus(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation);
			break;*/
	//	case "BotDeathMatch":
			theBot = BotDeathMatch(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
			//theBot = BotDeathMatchPlus(Level.Game).AddRemoteBot(self, clientName);
	//		break;
		/*case "BotDomination":
			theBot = BotDomination(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation);
			break;
		case "BotCTF":
			theBot = BotCTF(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation);
			break;	*/
	//}

	// broadcast a welcome message.
	//Level.Game.BroadcastMessage( theBot.PlayerReplicationInfo.PlayerName$Level.Game.EnteredMessage, false );	  // FIXME

	if(bDebug)
    		if(theBot != None) log("added bot, now running");
    		else log("ERROR ADDING BOT");

    gotoState('monitoring','Running');
}



//Closed on other end
event Closed()
{
    local Pawn tempPawn;

    if ( theBot != None && theBot.Pawn != None ) {
        theBot.SetLocation(theBot.Pawn.Location);
        theBot.Pawn.RemoteRole = ROLE_SimulatedProxy;
        theBot.Pawn.UnPossessed();
        tempPawn = theBot.Pawn;
        theBot.Pawn = None;
        tempPawn.Destroy();
    }

    if(theBot != None) {
        theBot.Destroy();
    }

    Destroy();
}

//Recieve info - parse into lines and call RecievedLine
event ReceivedText( string Text )
{
	local int i;
	local string S;

    if(bDebug)
    	log("Recieved - "$Text);

	ReceivedData = ReceivedData $ Text;

	// remove a LF which arrived in a new packet
	// and thus didn't get cleaned up by the code below
	if(Left(ReceivedData, 1) == Chr(10))
		ReceivedData = Mid(ReceivedData, 1);
	i = InStr(ReceivedData, Chr(13));
	while(i != -1)
	{
		S = Left(ReceivedData, i);
		i++;
		// check for any LF following the CR.
		if(Mid(ReceivedData, i, 1) == Chr(10))
			i++;

		ReceivedData = Mid(ReceivedData, i);

		ReceivedLine(S);

		if(LinkState != STATE_Connected)
			return;

		i = InStr(ReceivedData, Chr(13));
	}
}





//Here is where we handle incoming commands
/* Commands expected to look like:
{RUN {Argument value} {Arg value}...}
Currently had coded to take no more than 9 args
Command type and arguments can be
any length, but first space terminates the name. Values can
have spaces or any other kind of character.
*/
function ReceivedLine(string S)
{
	local string cmdType, argBody, rem;
	local int endloc, wordsplit;
    local name curBotState;

    if(bDebug)
    	log(S);

	if( theBot != none ) {
    	curBotState = theBot.GetStateName();
    	if( curBotState == 'Dying' || curBotState == 'GameEnded')
        	return;
	}

	wordsplit = InStr(S,ib);
	if( wordsplit == -1)
		wordsplit = Len(S);

	cmdType = left(S,wordsplit);
	rem = mid(S,InStr(S,as));
	attrNum = 0;

	//iterate through attr/val pairs, storring them in the
	//parallel arrays RecievedArgs and RecievedVals
	while(rem != "")
	{
		endloc = InStr(rem,ae);
		argBody = mid(rem,1,(endloc - 1));

		wordsplit = InStr(argBody,ib);
		ReceivedArgs[attrNum] = left(argBody,wordsplit);
		ReceivedVals[attrNum] = mid(argBody,(wordsplit + 1));

		rem = mid(rem,1); //advance
		rem = mid(rem,InStr(rem,as));
		attrNum++;
	}

   	cmdType = Caps(cmdType);

	ProcessAction(cmdType);
}


function string GetArgVal(string argName)
{
	local int i;
	while (i < attrNum && ReceivedArgs[i] != "")
	{
		if (ReceivedArgs[i] ~= argName)
			return ReceivedVals[i];
		i++;
	}

	return "";
}

function ParseVector(out vector v, string vecName)
{
    local int i;
    local string rem;
    local string delim;

    delim = ib;

	rem = GetArgVal(vecName);
	if(rem != "")
	{
        if( InStr(rem,delim) == -1 )
        	delim = ",";
        i = InStr(rem,delim);
        v.X = float(left(rem,i));
        rem = mid(rem,i+1);
        i = InStr(rem,delim);
        v.Y = float(left(rem,i));
        v.Z = float(mid(rem,i+1));
	}
	else
	{
		v.x = float( GetArgVal("x") );
		v.y = float( GetArgVal("y") );
		v.z = float( GetArgVal("z") );
	}
}

function ParseRotAsVec(out vector vec)
{
    local int i;
    local string rem;
    local string delim;

    delim = ib;

	rem = GetArgVal("Rotation");
	if(rem != "")
	{
        if( InStr(rem,delim) == -1 )
        	delim = ",";
        i = InStr(rem,delim);
        vec.X = float(left(rem,i));
        rem = mid(rem,i+1);
        i = InStr(rem,delim);
        vec.Y = float(left(rem,i));
        vec.Z = float(mid(rem,i+1));
	}
	else
	{
		vec.Y = float( GetArgVal("pitch") );
		vec.Z = float( GetArgVal("yaw") );
		vec.X = float( GetArgVal("roll") );
	}
}

function ParseRot(out rotator rot)
{
    local int i;
    local string rem;
    local string delim;

    delim = ib;

	rem = GetArgVal("Rotation");
	if(rem != "")
	{
        if( InStr(rem,delim) == -1 )
        	delim = ",";
        i = InStr(rem,delim);
        rot.Pitch = float(left(rem,i));
        rem = mid(rem,i+1);
        i = InStr(rem,delim);
        rot.Yaw = float(left(rem,i));
        rot.Roll = float(mid(rem,i+1));
	}
	else
	{
		rot.Pitch = float( GetArgVal("pitch") );
		rot.Yaw = float( GetArgVal("yaw") );
		rot.Roll = float( GetArgVal("roll") );
	}
}

function ProcessAction(string cmdType)
{
    local int i;
    local vector v, v2;
    local bool boolResult;
    local string target, id, outBuf, focusTarg;
    local rotator r, r2;
    local float speed;
    local float accel;

    if(bDebug)
		log("comandType:"@cmdType);

    if (bIterative)
    	Level.Pauser=None;


   	switch(cmdType)
   	{
	case "TEST":
	    theBot.GotoState('Startup', 'Test');
	    break;
	case "INIT":
	    InitRecieved();
	    break;
	case "INCH":
	    //test function
	    /* must deal with target, focus and destination */
	    theBot.StopWaiting();
	    theBot.myDestination = (100 * vector(theBot.Pawn.Rotation)) + theBot.Pawn.Location;
	    theBot.GotoState('Startup', 'Move');
	    break;
	case "SETWALK":
	    target = GetArgVal("Walk");
	    SetPropertyText("tempBool",target);
	    theBot.Pawn.bIsWalking = tempBool;
	    break;
	case "SETPHYS":
	    target = GetArgVal("Physics");
	    switch(target) {
	    case "Walking":
		theBot.Pawn.SetPhysics(PHYS_Walking);
		log("Setting Physics Walking");
		if(theBot.Physics == PHYS_Walking)
		    {
			log("Bot already walking");
		    }
		break;
	    case "Falling":
		theBot.Pawn.SetPhysics(PHYS_Falling);
		break;
	    case "Swimming":
		theBot.Pawn.SetPhysics(PHYS_Swimming);
		break;
	    case "Flying":
		theBot.Pawn.SetPhysics(PHYS_Flying);
		log("Setting Physics Flying");
		if(theBot.Pawn.Physics == PHYS_Flying)
		    {
			log("Bot already flying");
		    }
		break;
	    case "Rotating":
		theBot.Pawn.SetPhysics(PHYS_Rotating);
		break;
	    case "Projectile":
		theBot.Pawn.SetPhysics(PHYS_Projectile);
		break;
	    case "NONE":
	    case "":
		theBot.Pawn.SetPhysics(PHYS_None);
		break;
	    }
	    break;
	case "STOP":
	    theBot.GotoState('Startup', 'Begin');
	    break;
	case "JUMP":
	    theBot.RemoteJump();
	    break;
	case "DRM" : // May be we need to deal with time stamp to make time compensation. 02/06/17
	case "RUN" :
	case "SPIN": //jjwang 02/06/01
	    if	(theBot.botDR==none) {
		theBot.botDR = new class'DeadReckoning';
		theBot.botDR.init(theBot.Pawn);
	    }
	    theBot.botDR.SetT(Level.TimeSeconds);

	    //	SPIN
	    if (GetArgVal("Rotation")!="") {
		ParseRot(r);
		theBot.botDR.SetRP(r);
	    }
	    if (GetArgVal("Rate")!="") {
		ParseVector(v,"Rate");
		r.Pitch = v.x;
		r.Yaw = v.y;
		r.Roll = v.z;
		theBot.botDR.SetRV(r);
	    }
	    if (GetArgVal("AcRate")!="") {
		ParseVector(v,"AcRate");
		r.Pitch = v.x;
		r.Yaw = v.y;
		r.Roll = v.z;
		theBot.botDR.SetRA(r);
	    }

	    // RUN
	    if (GetArgVal("Location")!="") {
		ParseVector(v2,"Location");
		theBot.botDR.setLP(v2); // joe
		//theBot.Pawn.SetLocation(v2); //joe
	    }
	    if (GetArgVal("Velocity")!="") {
		ParseVector(v,"Velocity");
		theBot.botDR.setLV(v);
		theBot.botDR.setLA(vect(0,0,0));
	    }
	    if (GetArgVal("Acceleration")!="") {
		ParseVector(v,"Acceleration");
		theBot.botDR.SetLA(v);
	    }

	    // Used for replication DR. 02/06/27
	    theBot.start.DR_T = theBot.botDR.start.DR_T;
	    theBot.start.DR_P = theBot.botDR.start.DR_P;
	    theBot.start.DR_V = theBot.botDR.start.DR_V;
	    theBot.start.DR_A = theBot.botDR.start.DR_A;
	    theBot.start.DR_RP = theBot.botDR.start.DR_RP;
	    theBot.start.DR_RV = theBot.botDR.start.DR_RV;
	    theBot.start.DR_RA = theBot.botDR.start.DR_RA;
	    theBot.start.PreK = theBot.botDR.PreK;
	    theBot.start.bLFilter = theBot.botDR.bLFilter;
	    theBot.start.bRFilter = theBot.botDR.bRFilter;

	    if (!theBot.isDR) {
		log("STARTING DR");
		theBot.isDR = true;
		theBot.GotoState('Startup', 'DRM');
	    }
	    break;
	case "RUNTO":
	    speed = float(GetArgVal("Speed"));
	    if(speed != 0.0)
		{
		    if(theBot.Physics == PHYS_Flying)
			{
			    theBot.pawn.airspeed = speed;
			}
		    else if(theBot.Physics == PHYS_Swimming)
			{
			    theBot.pawn.waterspeed = speed;
			}
		    else
			{
			    theBot.pawn.groundspeed = speed;
			}
		}
	    accel = float(GetArgVal("Accel"));
	    if(accel != 0.0)
		{
		    theBot.pawn.AccelRate = accel;
		}
	    target = GetArgVal("Target");
	    if(target == "")
		{
		    ParseVector(v,"Location");
		    theBot.myDestination = v;
		    theBot.RemoteDestination = none;
		    theBot.GotoState('Startup', 'Move');
		}
	    else
		{
		    SetPropertyText("tempActor",target);

		    if( tempActor != none &&
			theBot.FastTrace(tempActor.Location) &&
			theBot.inFront(tempActor.Location) )
		    	{
			    //theBot.RemoteDestination = none;
			    //zero out myDestination
			    theBot.myDestination = tempActor.Location;
			    theBot.GotoState('Startup', 'Move');
			}
		}
	    break;
		case "STRAFE":
			speed = float(GetArgVal("Speed"));
                        if(speed != 0.0)
                        {
                                if(theBot.Pawn.Physics == PHYS_Flying)
                                {
                                        theBot.pawn.airspeed = speed;
                                }
				else if(theBot.Physics == PHYS_Swimming)
				{
					theBot.pawn.waterspeed = speed;
				}
                                else
                                {
                                        theBot.pawn.groundspeed = speed;
                                }
                        }
			accel = float(GetArgVal("Accel"));
			if(accel != 0.0)
			{
				theBot.pawn.AccelRate = accel;
			}
			ParseVector(v,"Location");
			theBot.myDestination = v;
			focusTarg = GetArgVal("Target");
			if(focusTarg == "")
			{
				ParseVector(v2,"Focus");
				theBot.myFocus = v2;
				theBot.RemoteFocus = none;
		    }
		    else
		    {
		    	SetPropertyText("tempActor",focusTarg);
		    	if( tempActor != none &&
		    		theBot.FastTrace(tempActor.Location) &&
		    		theBot.inFront(tempActor.Location) )
		    	{
			    	theBot.RemoteFocus = tempActor;
					//zero out myFocus
					theBot.myFocus = v2;
				}
		    }
			theBot.StopWaiting();
	        theBot.GotoState('Startup', 'Strafe');
	        break;
     	case "SHOOT":
			target = GetArgVal("Target");
			ParseVector(v,"Location");
			theBot.SetPropertyText("Enemy",target);
			theBot.SetPropertyText("RemoteEnemy",target);
			theBot.StopWaiting();
			theBot.myTarget = v;

			target = GetArgVal("Alt");
			SetPropertyText("tempBool",target);
			theBot.RemoteFireWeapon(tempBool);
	        break;
     	case "CHANGEWEAPON":
			target = GetArgVal("ID");
			if( target ~= "best" )
			{
				theBot.SwitchToBestWeapon();
			}
			else
			{
				SetPropertyText("tempActor",target);

				if(tempActor != none)
				{
					theBot.pawn.PendingWeapon = Weapon(tempActor);
					if ( theBot.pawn.Weapon == None )
						theBot.ChangedWeapon();
					else if ( theBot.pawn.Weapon != theBot.pawn.PendingWeapon )
						theBot.pawn.Weapon.PutDown();
				}
			}
	        break;
     	case "STOPSHOOT":
			theBot.StopWaiting();
			theBot.HaltFiring();
	        break;
     	case "CHECKREACH":
			target = GetArgVal("Target");
		    id = GetArgVal("id");
			if(target == "")
			{
				ParseVector(v,"Location");
		    	boolResult = theBot.pointReachable(v);
			    sendLine("RCH" $ib$as$ "ID" $ib$ id $ae$ib$as$ "Reachable" $ib$ boolResult
			    	$ae$ib$as$ "From" $ib$ theBot.Location $ae);
		    }
		    else
		    {
		    	SetPropertyText("tempActor",target);
		    	if( tempActor != none &&
		    		theBot.FastTrace(tempActor.Location) &&
		    		theBot.inFront(tempActor.Location) )
		    	{
			    	boolResult = theBot.actorReachable(tempActor);
				    sendLine("RCH" $ib$as$ "ID" $ib$ id $ae$ib$as$ "Reachable" $ib$ boolResult
				    	$ae$ib$as$ "From" $ib$ theBot.Location $ae$ib$as$ "To" $ib$ tempActor.Location $ae);
				}
		    }
    		break;
     	case "TURNTO":
	    target = GetArgVal("Target");
	    if(target == "")
		{
		    ParseRot(r);
		    if(r.Yaw == 0 && r.Pitch == 0 && r.Roll == 0)
			{
			    //no target or rotation defined
			    ParseVector(v,"Location");
			    theBot.myFocus = v;
			    theBot.RemoteFocus = none;
			}
		    else
			{
			  //no target, yes rotation
			  theBot.myRotation = r;
			  theBot.FocalPoint = theBot.Pawn.Location + ( vector(r) * 1000);
			  //theBot.myFocus = vect(0,0,0);
			  theBot.RemoteFocus = none;
		    	}
		}
	    else
		{
		    //target defined
		    SetPropertyText("tempActor",target);
		    if( tempActor != none &&
			theBot.FastTrace(tempActor.Location) &&
			theBot.inFront(tempActor.Location) )
		    	{
			    theBot.RemoteFocus = tempActor;
			    //zero out myFocus
			    theBot.myFocus = v;
			}
		}
	    if (GetArgVal("RotationRate")!="") {
                ParseRot(r2);
		theBot.myRotationRate = r2;
	    }
	    theBot.StopWaiting();
	    theBot.GotoState('Startup', 'Turning');
	    break;
     	case "ROTATE":
	    target = GetArgVal("Axis");
	    r = theBot.Pawn.Rotation;
	    i = int(GetArgVal("Amount"));
	    if(target == "Vertical")
		    {
			r.Pitch = r.Pitch + i;
		    }
		else
		    {
			r.Yaw = r.Yaw + i;
		    }
	        if (GetArgVal("RotationRate")!="") {
                    ParseRot(r2);
		    theBot.myRotationRate = r2;
	        }
		theBot.myRotation = r;
		theBot.myFocus = theBot.Pawn.Location + ( vector(r) * 1000 );
		theBot.RemoteFocus = none;
		theBot.StopWaiting();
		theBot.GotoState('Startup', 'Turning');
    		break;
	case "TRACK":
	    target = GetArgVal("Target");
	    if(target != "")
		{
		    SetPropertyText("tempActor",target);
		    if( theBot.FastTrace(tempActor.Location) && theBot.inFront(tempActor.Location) )
		    	{
			    theBot.RemoteFocus = tempActor;
			    theBot.StopWaiting();
			    theBot.GotoState('Startup', 'Tracking');
		        }
		}
	    break;
     	case "GETPATH":
     		//clear the old path
		    for ( i=0; i<16; i++ )
			{
				if ( theBot.RouteCache[i] == None )
					break;
				else
				{
					theBot.RouteCache[i] = None;
				}
			}
			ParseVector(v,"Location");
		    theBot.FindPathTo(v);
		    id = GetArgVal("id");
		    outBuf = "PTH"$ib$as$"ID"$ib$id$ae;
		    for ( i=0; i<16; i++ )
			{
				if ( theBot.RouteCache[i] == None )
					break;
				else
				{
    				outBuf = outBuf$ib$as$i$ib$theBot.RouteCache[i]$ib$theBot.RouteCache[i].Location$ae;
				}
			}
			SendLine(outBuf);
    		break;
     	case "MESSAGE":
     		//Note - currently only allow messages under 256 chars
			target = GetArgVal("Text");
			boolResult = bool(GetArgVal("Global"));
			if(target != "")
			{
				// @ preceding message makes it global. dont ask. not my code
		/*		if(boolResult)
					theBot.BroadcastLocalizedMessage("@"$target);
				else
					theBot.BroadcastLocalizedMessage(target); */
			// FIXME
		    }
		    break;
     	case "PING":
     		SendLine("PONG");
		    break;
    }
}




//Send a line to the client
function SendLine(string Text, optional bool bNoCRLF)
{
    if(bDebug)
    	log("    Sending: "$Text);
	if(bNoCRLF)
		SendText(Text);
	else
		SendText(Text$Chr(13)$Chr(10));
}


//Concat two strings togeather with our list bookends
function string MakeItem(string first, string second)
{
	return (as$first$ib$second$ae);
}


//fire right up into the loop for sending updates
auto state monitoring
{
Begin:
WaitingForInit:
	sleep(0.1);
	goto 'WaitingForInit';
Running:
  if(bVision && Level.Pauser == None && theBot != none && !theBot.IsInState('GameEnded') && !theBot.IsInState('Dying'))
  {
		SendLine("BEG" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
	 	//no clean way to do this since gametypes are not derived from
	 	//a common ancestor that has AddRemoteBot
		//switch(Parent.gameClass)
		//{
		/*	case "BotTeamGamePlus":
				sendLine( BotTeamGamePlus(Level.Game).GetGameStatus() );
				break;*/
		//	case "BotDeathMatch":
				sendLine( BotDeathMatch(Level.Game).GetGameStatus() );
		//		break;
			/*case "BotDomination":
				sendLine( BotDomination(Level.Game).GetGameStatus() );
				break;
			case "BotCTF":
				sendLine( BotCTF(Level.Game).GetGameStatus(theBot) );
				break;	*/
		//}
	  	theBot.checkSelf();
	  	theBot.checkVision();
		SendLine("END" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
  }
  if (bIterative)
  	Level.Pauser=theBot.playerReplicationInfo;

  sleep(visionTime);
  goto 'Running';
}

defaultproperties
{
     bDebug=true
     bVision=true
     visionTime=0.100000
     as="{"
     ae="}"
     ib=" "
}
