/* AIBO robot
 * by Marco Zaratti - marco.zaratti@gmail.com
*/
class ERS extends KRobot config(USARBot);

var int loAngleLimit[15];
var int hiAngleLimit[15];

function ProcessCarInput()
{
	local int nJoints;

	Super.ProcessCarInput();

	//Limit joints order 0 angles
	for(nJoints=0; nJoints < JointsControl.length; nJoints++)
		if((JointsControl[nJoints].state == 1) && (JointsControl[nJoints].order == 0))
		{
			if(JointsControl[nJoints].value < loAngleLimit[nJoints])
			{
				JointsControl[nJoints].value = loAngleLimit[nJoints];
				continue;
			}
			if(JointsControl[nJoints].value > hiAngleLimit[nJoints])
				JointsControl[nJoints].value = hiAngleLimit[nJoints];
		}
}

defaultproperties
{
    DrawScale=1
	bDebug=false
	MaxNetUpdateInterval=0.5

	//Low Joint Angle Limits
	loAngleLimit(0)=-21845
	loAngleLimit(1)=-2730
	loAngleLimit(2)=-5461
	loAngleLimit(3)=-24576
	loAngleLimit(4)=-2730
	loAngleLimit(5)=-5461
	loAngleLimit(6)=-21845
	loAngleLimit(7)=-2730
	loAngleLimit(8)=-5461
	loAngleLimit(9)=-24576
	loAngleLimit(10)=-2730
	loAngleLimit(11)=-5461
	loAngleLimit(12)=-14563
	loAngleLimit(13)=-16930
	loAngleLimit(14)=-3640
	//High Joint Angle Limits
	hiAngleLimit(0)=24576
	hiAngleLimit(1)=16930
	hiAngleLimit(2)=23119
	hiAngleLimit(3)=21845
	hiAngleLimit(4)=16930
	hiAngleLimit(5)=23119
	hiAngleLimit(6)=24576
	hiAngleLimit(7)=16930
	hiAngleLimit(8)=23119
	hiAngleLimit(9)=21845
	hiAngleLimit(10)=16930
	hiAngleLimit(11)=23119
	hiAngleLimit(12)=546
	hiAngleLimit(13)=16930
	hiAngleLimit(14)=9102

	StaticMesh=StaticMesh'spqrRobotMeshes.ERS7.ERS7Body'

    //Parameters of chassis. (KMass is defined in USAR.INI)
    Begin Object Class=KarmaParams Name=KParams0
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        bKNonSphericalInertia=true
        KActorGravScale=1
//Uncomment and define these when using RBfull karma params
//        KInertiaTensor(0)=0.4
//        KInertiaTensor(1)=0
//        KInertiaTensor(2)=0
//        KInertiaTensor(3)=0.4
//        KInertiaTensor(4)=0
//        KInertiaTensor(5)=0.4
//        KCOMOffset=(X=0.0,Y=0.0,Z=0.0)
        KMaxAngularSpeed=100
        KMaxSpeed=25000
        KLinearDamping=0.0
        KAngularDamping=0.0
        KFriction=0.8
        KRestitution=0.0
        Name="KParams0"
    End Object
    KParams=KarmaParams'KParams0'

//ERS7 scale 52.5:
/*
JointParts=(PartName="RFA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=3.4427,Y=2.1354,Z=-0.0918),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RFB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=1.176,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.3782,Y=0.2491,Z=-3.6938),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RRA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-3.4427,Y=2.1,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RRB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=1.05,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-0.5752,Y=0.2491,Z=-3.6684),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LFA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=3.4427,Y=-2.1354,Z=-0.0918),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LFB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-1.176,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.3782,Y=-0.2491,Z=-3.6938),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LRA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-3.4427,Y=-2.1,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LRB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-1.05,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-0.5752,Y=-0.2491,Z=-3.6684),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="HA",PartClass=Class'USARBot.ERSha',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=3.6027,Y=0.0,Z=0.9374),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HB",PartClass=Class'USARBot.ERShb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=4.2388),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="HC",PartClass=Class'USARBot.ERShc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
Sensors=(ItemClass=Class'USARBot.ERSirNear',Parent="HC",ItemName="IRN",Position=(X=4.4177,Y=0.21,Z=-0.21),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirFar',Parent="HC",ItemName="IRF",Position=(X=4.4177,Y=-0.21,Z=-0.21),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirEdge',Parent=,ItemName="EDG",Position=(X=6.0899,Y=0.0,Z=-0.3853),direction=(Y=-5461,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RFC",ItemName="RFP",Position=(X=1.5886,Y=0.0,Z=-2.6569),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LFC",ItemName="LFP",Position=(X=1.5886,Y=0.0,Z=-2.6569),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RRC",ItemName="RRP",Position=(X=-1.0407,Y=0.0,Z=-2.7703),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LRC",ItemName="LRP",Position=(X=-1.0407,Y=0.0,Z=-2.7703),direction=(Y=-16384,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.AccelerationSensor',Parent=,ItemName="ACC",Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HC",ItemName="BHS",Position=(X=3.99,Y=0.0,Z=-0.84),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HC",ItemName="Camera",Position=(X=3.99,Y=0.0,Z=-0.84),direction=(Y=0,Z=0,X=0))
Headlight=(ItemClass=None,Parent=,ItemName=,Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
*/
//ERS7 scale 250:
/*
JointParts=(PartName="RFA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=16.393999,Y=10.169,Z=-0.437),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RFB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=5.6,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=1.801,Y=1.1860,Z=-17.59),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RRA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-16.393999,Y=10.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RRB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=5.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-2.739,Y=1.186,Z=-17.469),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LFA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=16.393999,Y=-10.169,Z=-0.437),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LFB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-5.6,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=1.801,Y=-1.186,Z=-17.59),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LRA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-16.393999,Y=-10.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LRB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-5.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-2.739,Y=-1.186,Z=-17.469),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="HA",PartClass=Class'USARBot.ERSha',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=17.156,Y=0.0,Z=4.464),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HB",PartClass=Class'USARBot.ERShb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=20.184999),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="HC",PartClass=Class'USARBot.ERShc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
Sensors=(ItemClass=Class'USARBot.ERSirNear',Parent="HC",ItemName="IRN",Position=(X=21.037001,Y=1.0,Z=-1.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirFar',Parent="HC",ItemName="IRF",Position=(X=21.037001,Y=-1.0,Z=-1.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirEdge',Parent=,ItemName="EDG",Position=(X=29.0,Y=0.0,Z=-1.835),direction=(Y=-5461,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RFC",ItemName="RFP",Position=(X=7.565,Y=0.0,Z=-12.652),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LFC",ItemName="LFP",Position=(X=7.565,Y=0.0,Z=-12.652),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RRC",ItemName="RRP",Position=(X=-4.956,Y=0.0,Z=-13.192),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LRC",ItemName="LRP",Position=(X=-4.956,Y=0.0,Z=-13.192),direction=(Y=-16384,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.AccelerationSensor',Parent=,ItemName="ACC",Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HC",ItemName="BHS",Position=(X=19.0,Y=0.0,Z=-4.0),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HC",ItemName="Camera",Position=(X=19.0,Y=0.0,Z=-4.0),direction=(Y=0,Z=0,X=0))
Headlight=(ItemClass=None,Parent=,ItemName=,Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
*/
//ERS7 scale 80:
/*
JointParts=(PartName="RFA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=5.246,Y=3.254,Z=-0.14),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RFB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=1.792,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.576,Y=0.38,Z=-5.629),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RRA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-5.246,Y=3.2,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RRB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=1.6,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-0.876,Y=0.38,Z=-5.59),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LFA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=5.246,Y=-3.254,Z=-0.14),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LFB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-1.792,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.576,Y=-0.38,Z=-5.629),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LRA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-5.246,Y=-3.2,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LRB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-1.6,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-0.876,Y=-0.38,Z=-5.59),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="HA",PartClass=Class'USARBot.ERSha',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=5.49,Y=0.0,Z=1.428),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HB",PartClass=Class'USARBot.ERShb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=6.459),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="HC",PartClass=Class'USARBot.ERShc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
Sensors=(ItemClass=Class'USARBot.ERSirNear',Parent="HC",ItemName="IRN",Position=(X=6.732,Y=0.32,Z=-0.32),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirFar',Parent="HC",ItemName="IRF",Position=(X=6.732,Y=-0.32,Z=-0.32),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirEdge',Parent=,ItemName="EDG",Position=(X=9.28,Y=0.0,Z=-0.587),direction=(Y=-5461,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RFC",ItemName="RFP",Position=(X=2.421,Y=0.0,Z=-4.049),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LFC",ItemName="LFP",Position=(X=2.421,Y=0.0,Z=-4.049),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RRC",ItemName="RRP",Position=(X=-1.586,Y=0.0,Z=-4.221),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LRC",ItemName="LRP",Position=(X=-1.586,Y=0.0,Z=-4.221),direction=(Y=-16384,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.AccelerationSensor',Parent=,ItemName="ACC",Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HC",ItemName="BHS",Position=(X=6.08,Y=0.0,Z=-1.28),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HC",ItemName="Camera",Position=(X=6.08,Y=0.0,Z=-1.28),direction=(Y=0,Z=0,X=0))
Headlight=(ItemClass=None,Parent=,ItemName=,Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
*/
//ERS7 scale 500:
/*
JointParts=(PartName="RFA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=32.788,Y=20.338,Z=-0.874),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RFB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=11.2,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=3.602,Y=2.372,Z=-35.18),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="RRA",PartClass=Class'USARBot.ERSra',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-32.788,Y=20.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="RRB",PartClass=Class'USARBot.ERSrb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=10.0,Z=0.0),ParentAxis=(X=1.0),ParentAxis2=(Y=1.0),SelfAxis=(X=1.0),SelfAxis2=(Y=1.0))
JointParts=(PartName="RRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="RRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-5.478,Y=2.372,Z=-34.938),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LFA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=32.788,Y=-20.338,Z=-0.874),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LFB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-11.2,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LFC",PartClass=Class'USARBot.ERSfc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LFB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=3.602,Y=-2.372,Z=-35.18),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="LRA",PartClass=Class'USARBot.ERSla',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-32.788,Y=-20.0,Z=0.0),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="LRB",PartClass=Class'USARBot.ERSlb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=-10.0,Z=0.0),ParentAxis=(X=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(X=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="LRC",PartClass=Class'USARBot.ERSrc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="LRB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=-5.478,Y=-2.372,Z=-34.938),ParentAxis=(Y=1.0),ParentAxis2=(X=1.0),SelfAxis=(Y=1.0),SelfAxis2=(X=1.0))
JointParts=(PartName="HA",PartClass=Class'USARBot.ERSha',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent=,JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=34.312,Y=0.0,Z=8.928),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
JointParts=(PartName="HB",PartClass=Class'USARBot.ERShb',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HA",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=40.37),ParentAxis=(Z=-1.0),ParentAxis2=(Y=-1.0),SelfAxis=(Z=-1.0),SelfAxis2=(Y=-1.0))
JointParts=(PartName="HC",PartClass=Class'USARBot.ERShc',DrawScale3D=(X=1.0,Y=1.0,Z=1.0),Parent="HB",JointClass=Class'USARBot.KDHinge',BrakeTorque=50.0,ParentPos=(X=0.0,Y=0.0,Z=0.0),ParentAxis=(Y=-1.0),ParentAxis2=(X=-1.0),SelfAxis=(Y=-1.0),SelfAxis2=(X=-1.0))
Sensors=(ItemClass=Class'USARBot.ERSirNear',Parent="HC",ItemName="IRN",Position=(X=42.074,Y=2.0,Z=-2.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirFar',Parent="HC",ItemName="IRF",Position=(X=42.074,Y=-2.0,Z=-2.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.ERSirEdge',Parent=,ItemName="EDG",Position=(X=58.0,Y=0.0,Z=-3.67),direction=(Y=-5461,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RFC",ItemName="RFP",Position=(X=15.13,Y=0.0,Z=-25.304),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LFC",ItemName="LFP",Position=(X=15.13,Y=0.0,Z=-25.304),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="RRC",ItemName="RRP",Position=(X=-9.912,Y=0.0,Z=-26.384),direction=(Y=-16384,Z=0,X=0))
;Sensors=(ItemClass=Class'USARBot.ERSpaw',Parent="LRC",ItemName="LRP",Position=(X=-9.912,Y=0.0,Z=-26.384),direction=(Y=-16384,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.AccelerationSensor',Parent=,ItemName="ACC",Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
Sensors=(ItemClass=Class'USARBot.BallHSensor',Parent="HC",ItemName="BHS",Position=(X=38.0,Y=0.0,Z=-8.0),direction=(Y=0,Z=0,X=0))
Cameras=(ItemClass=Class'USARBot.AIBOCamera',Parent="HC",ItemName="Camera",Position=(X=38.0,Y=0.0,Z=-8.0),direction=(Y=0,Z=0,X=0))
Headlight=(ItemClass=None,Parent=,ItemName=,Position=(X=0.0,Y=0.0,Z=0.0),direction=(Y=0,Z=0,X=0))
*/
}
