// Dynamic Part
class KDPart extends KActor abstract;

// This should be an abstract and all of the parts of the robot should extend
// this class and define they own properties like StaticMesh, DrawScale3d, ...

// This is filled in by VehicleStateReceived in KRobot.
var KRigidBodyState ReceiveState;
var bool bReceiveStateNew;

// The type of the part, eg. ARM, LEG, Pan-Tilt etc.
var name Type;

var KConstraint     KJoint;
var config float Weight;

simulated function setJoint(KConstraint kc)
{
	KJoint = kc;
}

// This even is for updating the state (position, velocity etc.) of the tire's karma
// body when we get new information from the network.
simulated event bool KUpdateState(out KRigidBodyState newState)
{
    if(!bReceiveStateNew)
        return false;

    newState = ReceiveState;
    bReceiveStateNew = false;
    return true;
}

defaultproperties
{
    bNoDelete=false
    bStasis=True
    bAlwaysRelevant=True
    bRotateToDesired=True
    AmbientGlow=48  
    Type='Part'
    Mass=0.1

    bCollideActors=True
    CollisionHeight=8
    CollisionRadius=4

    StaticMesh=StaticMesh'USAR_Robots.Robots.TestBar'
    Begin Object Class=KarmaParamsRBFull Name=KParams0
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        KMaxAngularSpeed=100
        KMaxSpeed=25000
        KMass=0.1
    	KInertiaTensor(0)=0.4
        KInertiaTensor(1)=0
        KInertiaTensor(2)=0
        KInertiaTensor(3)=0.4
        KInertiaTensor(4)=0
        KInertiaTensor(5)=0.4
        KLinearDamping=0.0
        KAngularDamping=0.0
        KFriction=0.0
        KRestitution=0.3
        Name="KParams0"
    End Object
	KParams=KarmaParamsRBFull'KParams0'
	Weight=0.5
}
