class Rover extends KRobot config(USARBot);

var float cachedLeftValue,cachedRightValue;
var rotator cachedCameraRot,initCameraRot;
var bool bGetInitCameraRot;
var config bool bAbsoluteCamera;

var bool bAdjust;
var Quat expQ;
var config float safeDistance;


replication
{
	reliable if(Role == ROLE_Authority) expQ;
}

// The following simulate the differential of Rover. The differential ensures
// that the main body angle always averages the left and right wheel rocker angles.
simulated function Differential()
{
	local Quat relQ;
	local vector leftBarX,rightBarX,leftBarY,chassisX;
	local float diffAng;

	if (Role == ROLE_Authority) {
		leftBarX = QuatRotateVector(Parts[0].KGetRBQuaternion(),vect(1,0,0));
		leftBarY = QuatRotateVector(Parts[0].KGetRBQuaternion(),vect(0,1,0));
		rightBarX = QuatRotateVector(Parts[1].KGetRBQuaternion(),vect(1,0,0));
		if (VSize(leftBarX-rightBarX)<0.01) diffAng=0.0;
		else diffAng = -ACos(leftBarX Dot rightBarX)/2;
		chassisX = QuatRotateVector(QuatFromAxisAndAngle(leftBarY,diffAng),leftBarX);
		relQ = QuatFindBetween(QuatRotateVector(KGetRBQuaternion(),vect(1,0,0)),chassisX);
		expQ = QuatProduct(relQ,KGetRBQuaternion());
	}

	if (Role == ROLE_Authority)
		KGetRigidBodyState(ChassisState);

	if (ChassisState.Quaternion != expQ) {
		if (!KIsAwake()) KWake();
		ChassisState.Quaternion = expQ;
		bAdjust = true;
	}
}

// This only update the chassis. The wheels update themselves.
simulated event bool KUpdateState(out KRigidBodyState newState)
{
	// This should never get called on the server - but just in case!
	if(!bAdjust && (Role == ROLE_Authority || !bNewRobotState))
		return false;

	// Apply received data as new position of car chassis.
	newState = ChassisState;
	bNewRobotState = false;
	bAdjust = false;
	return true;
	//return false;
}

function ProcessCarInput()
{
	local float LeftValue,RightValue;
	local int CameraPan,CameraTilt,i;
	local bool bStop;

	Super.ProcessCarInput();

	if (!bGetInitCameraRot) {
		initCameraRot = myCamera.Rotation;
		USARRemoteBot(Controller).LeftThrottle = 0.0;
		USARRemoteBot(Controller).RightThrottle = 0.0;
		USARRemoteBot(Controller).myCameraRotation = rot(0,0,0);
		bGetInitCameraRot = true;
	}

	if (USARRemoteBot(Controller).bNewThrottle) {
	if (USARRemoteBot(Controller).Normalized) {
	LeftValue = USARRemoteBot(Controller).LeftThrottle*20000;
	RightValue = USARRemoteBot(Controller).RightThrottle*20000;
	} else {
		LeftValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).LeftThrottle);
		RightValue = Converter.SpinSpeedToUU(USARRemoteBot(Controller).RightThrottle);
	}

	if (cachedLeftValue!=LeftValue) {
		JointsControl[5].state = 1; // new command
		JointsControl[5].order = 1;
		JointsControl[5].value = LeftValue;
		JointsControl[7].state = 1; // new command
		JointsControl[7].order = 1;
		JointsControl[7].value = LeftValue;
		bNewCommand = true;
	}
	if (cachedRightValue!=RightValue) {
		JointsControl[4].state = 1; // new command
		JointsControl[4].order = 1;
		JointsControl[4].value = RightValue;
		JointsControl[6].state = 1; // new command
		JointsControl[6].order = 1;
		JointsControl[6].value = RightValue;
		bNewCommand = true;
	}
		if (JointsControl[5].value==JointsControl[7].value)
			cachedLeftValue=JointsControl[5].value;
		else
			cachedLeftValue=1000000; // a big value to force updating
		if (JointsControl[4].value==JointsControl[6].value)
			cachedRightValue=JointsControl[4].value;
		else
			cachedRightValue=1000000; // a big value to force updating
	}
	
	// turn
	if (abs(LeftValue)>1000 && LeftValue==-RightValue) {
		JointParts[0].BrakeTorque = MotorTorque*1.2;
		JointParts[1].BrakeTorque = MotorTorque*1.2;
		JointsControl[4].steer = -8000;
		JointsControl[5].steer = 8000;
		JointsControl[6].steer = 8000;
		JointsControl[7].steer = -8000;
	} else if (abs(LeftValue)>1000) {
		JointParts[0].BrakeTorque = 0;
		JointParts[1].BrakeTorque = 0;
		JointsControl[4].steer = 0;
		JointsControl[5].steer = 0;
		JointsControl[6].steer = 0;
		JointsControl[7].steer = 0;
	}
	if (myCamera!=None) {
		if (USARRemoteBot(Controller).myCameraRotation.roll!=0) return;
		USARRemoteBot(Controller).myCameraRotation.roll = -1;
		if (USARRemoteBot(Controller).myCameraRotationOrder==1) {
			CameraTilt = USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else if (!bAbsoluteCamera) {
			CameraTilt = -USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw;
		} else {
			CameraTilt = myCamera.Rotation.Pitch - initCameraRot.Pitch -Rotation.Pitch - USARRemoteBot(Controller).myCameraRotation.Pitch;
			CameraPan = USARRemoteBot(Controller).myCameraRotation.Yaw - myCamera.Rotation.Yaw + initCameraRot.Yaw + Rotation.Yaw;
			if (CameraTilt>65535) CameraTilt-=65535;
			else if (CameraTilt<-65535) CameraTilt+=65535;
			if (CameraTilt>32768) CameraTilt-=65535;
			else if (CameraTilt<-32768) CameraTilt+=65535;
			if (CameraPan>65535) CameraPan-=65535;
			else if (CameraPan<-65535) CameraPan+=65535;
			if (CameraPan>32768) CameraPan-=65535;
			else if (CameraPan<-32768) CameraPan+=65535;
		}
		if (CameraPan!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[10].state = 1;
			//JointsControl[10].steer = 65535;
			JointsControl[10].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[10].value = CameraPan;
		}
		if (JointsControl[10].order==1)
			PanSpeed = CameraPan;
		else
			PanSpeed = MotorSpeed;
		if (CameraTilt!=0 || USARRemoteBot(Controller).myCameraRotationOrder==1) {
			JointsControl[11].state = 1;
			//JointsControl[11].steer = 65535;
			JointsControl[11].order = USARRemoteBot(Controller).myCameraRotationOrder;
			JointsControl[11].value = CameraTilt;
		}
		if (JointsControl[11].order==1)
			TiltSpeed = CameraTilt;
		else
			TiltSpeed = MotorSpeed;
		bNewCommand = true;
		//restrict the camera tilting angle
		if (abs(myCamera.Rotation.Pitch - initCameraRot.Pitch - JointsControl[11].value)>11000) {
			JointsControl[11].value=0;
			JointsControl[11].state=0;
			KCarWheelJoint(Joints[11]).KMotorTorque = 0.0;
			KCarWheelJoint(Joints[11]).KBraking = 1.0;
			KCarWheelJoint(Joints[11]).KMaxSpeed = 1310700;
		}
	}
	// range sensor guard. If close to wall, stop me
	bStop=false;
	if (RangeSensor(SensorList[0]).GetRange()<SafeDistance) {
		for (i=4; i<=7; i++)
			if (JointsControl[i].order!=1 || JointsControl[i].value>0.0) {
				bStop=true;
				break;
			}
		if (bStop) {
			log("blocking...");
	 		JointParts[0].BrakeTorque = 0;
			JointParts[1].BrakeTorque = 0;
			for (i=4; i<=7; i++) {
				JointsControl[i].state = 1; // new command
				JointsControl[i].order = 1;
				JointsControl[i].value = 0.0;
			}
		}
	}
	//bReady = true;
}

simulated function Tick(float Delta)
{
	local int i;
	local bool bStop;

	Super.Tick(Delta);
	Differential();
	// range sensor guard. If close to wall, stop me
	bStop=false;
	if (RangeSensor(SensorList[0]).GetRange()<SafeDistance) {
		for (i=4; i<=7; i++)
			if (JointsControl[i].order!=1 || JointsControl[i].value>0.0) {
				bStop=true;
				break;
			}
		if (bStop) {
			log("blocking...");
	 		JointParts[0].BrakeTorque = 0;
			JointParts[1].BrakeTorque = 0;
			for (i=4; i<=7; i++) {
				JointsControl[i].state = 1; // new command
				JointsControl[i].order = 1;
				JointsControl[i].value = 0.0;
			}
		}
	}
}


defaultproperties
{
	bDebug=false
	safeDistance=0.18
	ChassisMass=1.000000
	StaticMesh=StaticMesh'USAR_Robots.Robots.PERBody'
	//StaticMesh=StaticMesh'USAR_Meshes2003.Robot.PERBody'
	DrawScale=1.0
	DrawScale3D=(X=1.0,Y=1.0,Z=1.0)

	TireRestitution=0.000000
	TireSoftness=0.000001
	TireRollFriction=15.000000
	TireLateralFriction=15.00000
	TireSlipRate=0.0002000
	MotorTorque=25.0

    Begin Object Class=KarmaParamsRBFull Name=KParams0
        KInertiaTensor(0)=0.010000
        KInertiaTensor(3)=0.02000
        KInertiaTensor(5)=0.03000
        KCOMOffset=(X=0.000000,Z=0.000000)
        KLinearDamping=0.500000
        KAngularDamping=50.00000
        KStartEnabled=True
        bHighDetailOnly=False
        bClientOnly=False
        bKDoubleTickRate=True
        KFriction=1.600000
        Name="KParams0"
    End Object
    KParams=KarmaParamsRBFull'USARBot.Rover.KParams0'
}
