/*
    This script allows to move the ball around the map. SetLocation function
    is not enough becouse it updates only the mesh position, note the rigid
    body state. That's why we need to update manually rigid body state. Here's
    another problem: if we move the ball near the previous position (100~200uu)
    than movement will not be precise. It seems that Karma applies an impulse to
    the ball. Only if we move the ball far away the position is correctly updated.
    I don't know the reason of this behavior, for now I'll hack movement code
    making a 2 step movement: 1) I'll move the ball far away both from current and
    destination position. 2) I'll move the ball to destination position.

     by Marco Zaratti - marco.zaratti@gmail.com
*/
class SoccerBall extends KNActor config(USARBot);

var KRigidBodyState BallRB;
var int BallState; //0: normal, 1: ready to move, 2: in "space"
var vector newPos, tempPos;

//Kick variables
var vector kickDirection;
var config float kickForce;

simulated event bool KUpdateState(out KRigidBodyState newState)
{
    switch(BallState)
    {
        case 0:
            return false;
        case 1:
            newState = BallRB;
            BallRB.Position = KRBVecFromVector(newPos);
            BallRB.AngVel = KRBVecFromVector(vect(0,0,0));
            BallRB.LinVel = KRBVecFromVector(vect(0,0,0));
            BallState++;
            break;
        case 2:
            newState = BallRB;
            BallState = 0;
            break;
    }

	return true;
}

simulated function MoveBall(float posX, float posY, float posZ)
{
    newPos.X = posX; newPos.Y = posY; newPos.Z = posZ;

    KGetRigidBodyState(BallRB);
    BallRB.Position = KRBVecFromVector(tempPos);
    BallRB.AngVel = KRBVecFromVector(vect(0,0,0));
    BallRB.LinVel = KRBVecFromVector(vect(0,0,0));
    BallState++;
}

simulated function KickBall(String direction, float force)
{
    local vector dest;

    if(direction ~= "Y")
        dest.X = -670;
    else if(direction ~= "B")
        dest.X = 670;
    //other else if....
    else return;

    //force and kickForce are exponents
    if(force ~=0)
        force = kickForce;

    force = 150 * exp(force/1.44);    //force ~= 150* 2^force
    dest.Y = RandRange(-100,100);
    kickDirection = Normal(dest - location) * force;

    KGetRigidBodyState(BallRB);
    BallRB.LinVel = KRBVecFromVector(kickDirection);
    BallState = 2;
}

defaultproperties
{
    bNoDelete=false
    kickForce=2
    tempPos=(X=0,Y=0,Z=900) //Temporary position for ball movement
	StaticMesh=StaticMesh'spqrMapsMeshes.AiboSoccer.Ball_8cm'
    Begin Object Class=KarmaParams Name=KBall
        KStartEnabled=True
        bKDoubleTickRate=True
        bHighDetailOnly=False
        bClientOnly=False
        KMass=0.04
        KLinearDamping=0.15
        KAngularDamping=0.1
        KMaxAngularSpeed=100
        KMaxSpeed=25000
        KFriction=0.05
        KRestitution=0.9
        Name="KBall"
    End Object
    KParams=KarmaParams'USARBot.SoccerBall.KBall'
}
