class USARBc extends KTwoWheels placeable config(USARBot);

var float LeftThrottle;
var float RightThrottle;
var rotator HLRotation;
var int CameraZoom;
var float PanSpeed, TiltSpeed;
var config float CameraDefFov, CameraMinFov, CameraMaxFov;
var float degCameraDefFov, degCameraMinFov, degCameraMaxFov;

// headlight projector
var const vector		HeadlightOffset;
var USARHeadlight	Headlight;
var bool				HeadlightOn;
var byte	HeadlightItensity;

// Wheel Draw Scale
var float TireDrawScale;
var vector TireDrawScale3D;

// Wheel dirt emitter
var BulldogDust		Dust[2]; // FL, FR

// Distance below centre of tire to place dust.
var float			DustDrop;

// Ratio between dust kicked up and amount wheels are slipping
var (USARCar) float	DustSlipRate;
var (USARCar) float DustSlipThresh;

var (USARCar) Material	TargetMaterial;

// USARCar sound things
var (USARCar) float	EnginePitchScale;
var (USARCar) float	HitSoundThreshold;
var config bool bDebug;

// USAR Message
var config float msgTimer;

// USAR Sensors
struct SSensor {
	var class<Sensor> SensorClass;
	var string SenName;
	var vector Position;
	var vector Direction;
	var rotator uuDirection;
};
var config array<SSensor> Sensors;
var array<Sensor> SensorList;
var array<byte>	ProcessedSensors;

// USAR Battery
var config int batteryLife;
var int startTime;
var int myLife;

var config bool bSpeedControl;
var config bool bAbsoluteCamera;
var vector oldPos;
var byte myState;

// pid control loop
var config float Kp,Ki,Kd;
var config float MinOut,MaxOut;

var float pidErr;
var float pidInt;
var float pidDer;
var float lastErr;

var float pidErr2;
var float pidInt2;
var float pidDer2;
var float lastErr2;

var config string ConverterClass;
var USARConverter converter;
var config bool   bMountByUU;
var config bool   bDisplayTeamBeacon;

replication
{
	unreliable if(Role == ROLE_Authority)
		LeftThrottle, RightThrottle, HLRotation, HeadlightOn, HeadLightItensity, CameraZoom, myState;
}

simulated event PreBeginPlay()
{
	local class<USARConverter> cClass;

	Super.PreBeginPlay();
	bNoTeamBeacon=!bDisplayTeamBeacon;
	cClass = class<USARConverter>(DynamicLoadObject(ConverterClass, class'Class'));
	converter = new cClass;
	ConvertParam(converter);
}

simulated function ConvertParam(USARConverter converter)
{
    local int i;

    if (converter==None) {
    	degCameraDefFov = CameraDefFov;
    	degCameraMinFov = CameraMinFov;
    	degCameraMaxFov = CameraMaxFov;
    } else {
        degCameraDefFov = converter.AngleToDeg(CameraDefFov);
        degCameraMinFov = converter.AngleToDeg(CameraMinFov);
        degCameraMaxFov = converter.AngleToDeg(CameraMaxFov);
    }
    if (!bMountByUU && converter!=None) {
       	for (i=0;i<Sensors.length;i++) {
		    Sensors[i].Position = converter.LengthVectorToUU(Sensors[i].Position);
		    Sensors[i].uuDirection = converter.RotatorToUU(Sensors[i].Direction);
		}
    } else {
       	for (i=0;i<Sensors.length;i++) {
		    Sensors[i].uuDirection.Roll  = int(Sensors[i].Direction.X);
		    Sensors[i].uuDirection.Pitch = int(Sensors[i].Direction.Y);
		    Sensors[i].uuDirection.Yaw   = int(Sensors[i].Direction.Z);
		}
    }
}

// Register to the game
function Register()
{
	local USARDeathMatch UsarGame;
	local int Index;

	UsarGame = USARDeathMatch(Level.Game);
	Index = UsarGame.Vehicles.length;
	UsarGame.Vehicles.Insert(Index,1);
	UsarGame.Vehicles[Index]=self;
}

// Unregister from the game
function Unregister()
{
	local USARDeathMatch UsarGame;
	local int i;

	UsarGame = USARDeathMatch(Level.Game);
	for (i=0;i<UsarGame.Vehicles.length;i++) {
		if (UsarGame.Vehicles[i]==self) {
			UsarGame.Vehicles.Remove(i,1);
			break;
		}
	}
}

//
//	The following functions are simplified to simulate USAR Robot.
//  1) No Brake 2) No Steer
//			by jjwang 03/03/03
//
simulated function PostNetBeginPlay()
{
	local vector RotX, RotY, RotZ;
	local int i;

    Super.PostNetBeginPlay();
    GetAxes(Rotation,RotX,RotY,RotZ);

    frontLeft.SetDrawScale3D(TireDrawScale3D);
    frontRight.SetDrawScale3D(TireDrawScale3D*vect(1,-1,1));

	HLRotation = rot(0,0,0);
	Headlight = spawn(class'USARHeadlight', self,, Location + HeadlightOffset.X * RotX + HeadlightOffset.Y * RotY + HeadlightOffset.Z * RotZ);
	Headlight.SetBase(self);
	Headlight.SetRelativeRotation(HLRotation);
	CameraZoom = degCameraDefFov;

	for (i=0;i<Sensors.length;i++) {
		SensorList[i] = spawn(Sensors[i].SensorClass,
						      self,,
						      Location + Sensors[i].Position.X * RotX + Sensors[i].Position.Y * RotY + Sensors[i].Position.Z * RotZ,
						      );
		SensorList[i].init(Sensors[i].SenName,self,Sensors[i].Position,Sensors[i].uuDirection,self,'None');
		ProcessedSensors[i] = 0;
	}

	if(Level.NetMode != NM_DedicatedServer)
	{
		// Create wheel dust emitters.
		Dust[0] = spawn(class'BulldogDust', self,, Location + WheelFrontAlong*RotX + WheelFrontAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[0].SetBase(self);
		Dust[0].SetDrawScale(0.21);

		Dust[1] = spawn(class'BulldogDust', self,, Location + WheelFrontAlong*RotX - WheelFrontAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[1].SetBase(self);
		Dust[1].SetDrawScale(0.21);
	}

    // For KImpact event
    KSetImpactThreshold(HitSoundThreshold);

	// If this is not 'authority' version - don't destroy it if there is a problem.
	// The network should sort things out.
	if(Role != ROLE_Authority)
	{
		KarmaParams(KParams).bDestroyOnSimError = False;
		KarmaParams(frontLeft.KParams).bDestroyOnSimError = False;
		KarmaParams(frontRight.KParams).bDestroyOnSimError = False;
	}

	Register();
	SetTimer(msgTimer,true);
	startTime = Level.TimeSeconds;
	Enable('Tick');
}

simulated event Destroyed()
{
	local int i;
	//Log("Bulldog Destroyed");

	if(Level.NetMode != NM_DedicatedServer)
	{
		Dust[0].Destroy();
		Dust[1].Destroy();
	}

	for (i=0;i<Sensors.length;i++)	SensorList[i].Destroy();
	Headlight.Destroy();
	converter = None;

	Unregister();
	// This will destroy wheels & joints
	Super.Destroyed();
}

// For drawing targetting reticule
simulated function DrawHud(Canvas C)
{
	//local vector ScreenPos;

	if (myLife==batteryLife) {
		C.Reset();
		C.Style = ERenderStyle.STY_Particle;
		C.SetPos(0,0);
		C.DrawTile(Material'XGameShaders.ScreenNoisePan', C.SizeX, C.SizeY, 0.0, 0.0, 128, 128);
		C.FontScaleX=4;
		C.FontScaleY=4;
		C.DrawColor.R = 255;
		C.DrawColor.G = 0;
		C.DrawColor.B = 0;
		C.DrawColor.A = 255;
		C.SetPos(C.SizeX/2-80, C.SizeY/2);
		C.DrawText("NO POWER!");
		return;
	}
	if (bDebug) {
		C.Style = ERenderStyle.STY_Additive;
		C.FontScaleX=2;
		C.FontScaleY=2;
		C.DrawColor.R = 255;
		C.DrawColor.G = 128;
		C.DrawColor.B = 128;
		C.DrawColor.A = 255;
		C.SetPos(0, 0);
		C.DrawText("Left"@LeftThrottle@frontLeft.SpinSpeed@frontLeft.WheelJoint.KMotorTorque);
		C.DrawColor.R = 128;
		C.DrawColor.G = 255;
		C.SetPos(0, 30);
		C.DrawText("Right"@RightThrottle@frontRight.SpinSpeed@frontRight.WheelJoint.KMotorTorque);
	}
}

simulated event KVehicleUpdateParams()
{
    if (frontLeft==NONE) return;
    Super.KVehicleUpdateParams();

    frontLeft.WheelJoint.bKSteeringLocked = true;
    frontRight.WheelJoint.bKSteeringLocked = true;

	frontLeft.WheelJoint.KUpdateConstraintParams();
	frontRight.WheelJoint.KUpdateConstraintParams();
}

function float myEngine(float Throttle, float SpinSpeed)
{
    local float torque;

	torque = Throttle * InterpCurveEval(TorqueCurve, SpinSpeed);

	GraphData("SpinSpeed", SpinSpeed);
	GraphData("Torque", torque);

    return -1 * torque;
}

//////////////////////////////////////////////
// pid control loop
// W(s) = Kp +  Ki*(1/s) + KD*s
//////////////////////////////////////////////
function float pidEngine(float desiredPV, float currentPV, float delta)
{
	local float COut;

	pidErr = desiredPV - currentPV;

	pidInt = pidInt + pidErr * delta;
	pidDer = (pidErr - lastErr) / delta;
	lastErr = pidErr;

	COut = Kp*pidErr+pidInt*Ki+pidDer*Kd;

	if (COut<MinOut) COut=MinOut;
	else if (COut>MaxOut) COut=MaxOut;

	return -1 * COut;
}

function float pidEngine2(float desiredPV, float currentPV, float delta)
{
	local float COut;

	pidErr2 = desiredPV - currentPV;
	pidInt2 = pidInt2 + pidErr2 * delta;
	pidDer2 = (pidErr2 - lastErr2) / delta;
	lastErr2 = pidErr2;

	COut = Kp*pidErr2+pidInt2*Ki+pidDer2*Kd;

	if (COut<MinOut) COut=MinOut;
	else if (COut>MaxOut) COut=MaxOut;

	return -1 * COut;
}

function StartFlip(Pawn Pusher)
{
	//local vector toPusher, worldUp;

	// if we are already flipping the car - dont do it again!
	if(FlipTimeLeft > 0)
		return;

	FlipTimeLeft = FlipTime; // Start the flip on the server
	USARRemoteBot(Controller).Flip = false;
}

function ProcessCarInput()
{
	local vector worldForward, worldUp;
	local string outstring, geoType, confType;
	local int i, Jid;

	// Return sensor geo info
	geoType = USARRemoteBot(Controller).GeoType;
	if (geoType!="") {
		outstring = "";
		for (i=0;i<Sensors.length;i++) {
			if (SensorList[i].isType(geoType)) {
				outstring = outstring@SensorList[i].GetGeoData();
				Jid=i;
			}
		}
		USARRemoteBot(Controller).myConnection.SendLine(SensorList[Jid].GetGeoHead()$outstring);
		USARRemoteBot(Controller).GeoType="";
	}
	// Return sensor conf info
	confType = USARRemoteBot(Controller).ConfType;
	if (confType!="") {
		outstring="";
		for (i=0;i<Sensors.length;i++) {
			if (SensorList[i].isType(confType)) {
				outstring = outstring@SensorList[i].GetConfData();
				Jid=i;
			}
		}
		if (outstring!="")
			USARRemoteBot(Controller).myConnection.SendLine(SensorList[Jid].GetConfHead()$outstring);
		USARRemoteBot(Controller).ConfType="";
	}

	//Log("PCI S:"$Steering$" T:"$Throttle);

  	worldForward = vect(-1, 0, 0) >> Rotation;
  	worldUp = vect(0, 0, 1) >> Rotation;

  	ForwardVel = Velocity Dot worldForward;

	bIsInverted = worldUp.Z < 0.2;

	// 'ForwardVel' isn't very helpful if we are inverted, so we just pretend its positive.
	if(bIsInverted)
		ForwardVel = 2 * StopThreshold;

	//if (abs(LeftThrottle)<0.01&&abs(RightThrottle)<0.01&&(frontLeft.SpinSpeed>StopThreshold||frontRight.SpinSpeed>StopThreshold))
	if (abs(LeftThrottle)<0.01&&abs(RightThrottle)<0.01)
		OutputBrake = true;
	else OutputBrake = false;
	IsDriving = true;
	OutputHandbrakeOn = false;
	KWake(); // currently, never let the car go to sleep whilst being driven.
}

// Car Simulation
simulated function Tick(float Delta)
{
    local float EnginePitch;
	local float actVel;

    // Battery
	if (myLife>=batteryLife) {
		myLife=batteryLife;
		USARRemoteBot(Controller).LeftThrottle=0;
		USARRemoteBot(Controller).RightThrottle=0;
		USARRemoteBot(Controller).myTurn.roll=0;
		USARRemoteBot(Controller).Light=false;
	}
	else
		myLife = Level.TimeSeconds - startTime;

    WheelSpinSpeed = (frontLeft.SpinSpeed + frontRight.SpinSpeed)/2;
    //log("WheelSpinSpeed:"$WheelSpinSpeed);

    // if we are in the process of flipping the car, keep it enabled!
    if( FlipTimeLeft > 0  )	KWake();

    // If the server, process input and pack updated car info into struct.
    if(Role == ROLE_Authority) {
		ProcessCarInput();
		PackState();
		if (bSpeedControl && myState==0) {
			if (LeftThrottle != USARRemoteBot(Controller).LeftThrottle*6) pidInt=0;
			LeftThrottle = USARRemoteBot(Controller).LeftThrottle*6;
			if (RightThrottle != USARRemoteBot(Controller).RightThrottle*6) pidInt2=0;
			RightThrottle = USARRemoteBot(Controller).RightThrottle*6;
		}
		else {
			LeftThrottle = USARRemoteBot(Controller).LeftThrottle;
			RightThrottle = USARRemoteBot(Controller).RightThrottle;
		}
		if (LeftThrottle*RightThrottle>0) {
			LeftThrottle = -LeftThrottle;
			RightThrottle = -RightThrottle;
		}
		Throttle = (LeftThrottle+RightThrottle)/2.0;

		// If new 'drive' command is coming, stop the current tunring state.
		if ( (bSpeedControl && (LeftThrottle != USARRemoteBot(Controller).LeftThrottle*30 || RightThrottle != USARRemoteBot(Controller).RightThrottle*30))
		     ||
		     (!bSpeedControl && (LeftThrottle != USARRemoteBot(Controller).LeftThrottle || RightThrottle != USARRemoteBot(Controller).RightThrottle)) ) {
			myState=0;
			USARRemoteBot(Controller).myTurn.roll=0; // Stop the turning action
		}

		if (USARRemoteBot(Controller).myTurn.roll>0) {
			if (abs(USARRemoteBot(Controller).myTurn.Yaw-Rotation.Yaw)<200 || myState==2) {
				if (USARRemoteBot(Controller).myTurn.pitch>=0) {
					USARRemoteBot(Controller).myTurn.pitch=-10;
					oldPos=Location;
				}
				if (abs(vsize(Location-oldPos)-USARRemoteBot(Controller).myDistance)<50) {
					USARRemoteBot(Controller).myTurn.roll=-10;
					LeftThrottle=0;
					RightThrottle=0;
					myState=0;
				}
				else {
					myState=2;
					LeftThrottle=5;
					RightThrottle=5;
				}
			}
			else {
				myState=1;
				if (USARRemoteBot(Controller).myTurn.Yaw>0) {
					LeftThrottle=5;
					RightThrottle=-5;
				}
				else {
					LeftThrottle=-5;
					RightThrottle=5;
				}
			}
		}
		else myState=0;
	}

	if (bSpeedControl||myState==2||myState==1) {
		//Left
		actVel = frontLeft.Velocity dot (vect(-1, 0, 0) >> Rotation);
		if (actVel>0.01||actVel<-0.01||LeftThrottle>0.5||LeftThrottle<-0.5)
			frontLeft.WheelJoint.KMotorTorque=pidEngine(LeftThrottle,actVel,Delta);
		else
			frontLeft.WheelJoint.KMotorTorque=0.0;

		if (LeftThrottle*(LeftThrottle-actVel)>0) {
			if (LeftThrottle>0&&frontLeft.WheelJoint.KMotorTorque>0)
				frontLeft.WheelJoint.KMotorTorque=-0.01;
			if (LeftThrottle<0&&frontLeft.WheelJoint.KMotorTorque<0)
				frontLeft.WheelJoint.KMotorTorque=0.01;
		}
		if (frontLeft.WheelJoint.KMotorTorque>40) frontLeft.WheelJoint.KMotorTorque=40;
		if (frontLeft.WheelJoint.KMotorTorque<-40) frontLeft.WheelJoint.KMotorTorque=-40;

   		if (bDebug) log("-LEFT DV:"$LeftThrottle@" V:"$actVel@" Er:"$pidErr@" IER:"$pidInt@" Out:"$frontLeft.WheelJoint.KMotorTorque);

		//Right
		actVel = frontRight.Velocity dot (vect(-1, 0, 0) >> Rotation);
		if (actVel>0.01||actVel<-0.01||RightThrottle>0.5||RightThrottle<-0.5)
			frontRight.WheelJoint.KMotorTorque=pidEngine2(RightThrottle,actVel,Delta);
		else
			frontRight.WheelJoint.KMotorTorque=0.0;

		if (RightThrottle*(RightThrottle-actVel)>0) {
			if (RightThrottle>0&&frontRight.WheelJoint.KMotorTorque>0)
				frontRight.WheelJoint.KMotorTorque=-0.01;
			if (RightThrottle<0&&frontRight.WheelJoint.KMotorTorque<0)
				frontRight.WheelJoint.KMotorTorque=0.01;
		}
		if (frontRight.WheelJoint.KMotorTorque>40) frontRight.WheelJoint.KMotorTorque=40;
		if (frontRight.WheelJoint.KMotorTorque<-40) frontRight.WheelJoint.KMotorTorque=-40;

   		if (bDebug) log("-RIGHT DV:"$RightThrottle@" V:"$actVel@" Er:"$pidErr2@" IER:"$pidInt2@" Out:"$frontRight.WheelJoint.KMotorTorque);

    }else {
    	frontLeft.WheelJoint.KMotorTorque = myEngine(LeftThrottle,frontLeft.SpinSpeed);
    	frontRight.WheelJoint.KMotorTorque = myEngine(RightThrottle,frontRight.SpinSpeed);
    }

    // Braking
	if(OutputBrake)
	{
		frontLeft.WheelJoint.KBraking = MaxBrakeTorque;
		frontRight.WheelJoint.KBraking = MaxBrakeTorque;
	}
	else
	{
		frontLeft.WheelJoint.KBraking = 0.0;
		frontRight.WheelJoint.KBraking = 0.0;
	}

	if(OutputHandbrakeOn == true)
	{
		//Log("HANDBRAKE!!");
		frontLeft.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
		frontLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

		frontRight.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
		frontRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
	}
	else
	{
		frontLeft.LateralFriction = TireLateralFriction;
		frontLeft.LateralSlip = TireLateralSlip;

		frontRight.LateralFriction = TireLateralFriction;
		frontRight.LateralSlip = TireLateralSlip;
	}

	// Flipping
	if(FlipTimeLeft > 0)
	{
		FlipTimeLeft -= Delta;
		FlipTimeLeft = FMax(FlipTimeLeft, 0.0); // Make sure it doesn't go negative
	}

	if (Role==ROLE_Authority) {

	    // log("*****PITCH:" $USARRemoteBot(Controller).uuCameraRotation.pitch $ " - " $ HLRotation.pitch);
	    //log("*****YAW:" $USARRemoteBot(Controller).uuCameraRotation.yaw $ " - " $ HLRotation.yaw);
	    if (USARRemoteBot(Controller).myCameraRotationOrder==0) {
	    	if (!bAbsoluteCamera&&USARRemoteBot(Controller).myCameraRotation.roll==0) {
	    		USARRemoteBot(Controller).myCameraRotation+=HLRotation;
	    		USARRemoteBot(Controller).myCameraRotation.roll=-1;
	    	}
			if (abs(HLRotation.pitch - USARRemoteBot(Controller).myCameraRotation.pitch) > 100.00) {
		    	if (USARRemoteBot(Controller).myCameraRotation.pitch > HLRotation.pitch)
					HLRotation.pitch += 100.00;
		    	else if (USARRemoteBot(Controller).myCameraRotation.pitch < HLRotation.pitch)
					HLRotation.pitch -= 100.00;
		    	else
					HLRotation.pitch += 0.0;
			}
			else HLRotation.pitch = USARRemoteBot(Controller).myCameraRotation.pitch;

			if (abs(HLRotation.yaw - USARRemoteBot(Controller).myCameraRotation.yaw) > 100.00) {
		    	if (USARRemoteBot(Controller).myCameraRotation.yaw > HLRotation.yaw)
					HLRotation.yaw += 100.00;
		    	else if (USARRemoteBot(Controller).myCameraRotation.yaw < HLRotation.yaw)
					HLRotation.yaw -= 100.00;
		    	else
					HLRotation.yaw += 0.0;
			}
			else HLRotation.yaw = USARRemoteBot(Controller).myCameraRotation.yaw;
			PanSpeed = 100.0/Delta;
			TiltSpeed = 100.0/Delta;
		}else {
			HLRotation+=USARRemoteBot(Controller).myCameraRotation;
			PanSpeed = USARRemoteBot(Controller).myCameraRotation.Yaw/Delta;
			TiltSpeed = USARRemoteBot(Controller).myCameraRotation.Pitch/Delta;
		}

	    /*
		r.Pitch=(USARRemoteBot(Controller).myCameraRotation).Pitch+default.HLRotation.Pitch;
		r.Yaw=HLRotation.Yaw+(USARRemoteBot(Controller).myCameraRotation).Yaw;
		r.Roll=HLRotation.Roll+(USARRemoteBot(Controller).myCameraRotation).Roll;
		HLRotation=r;
		*/
		HeadlightOn=USARRemoteBot(Controller).Light;
		if (USARRemoteBot(Controller).Flip) StartFlip(None);

		if (USARRemoteBot(Controller).myCameraZoom>=0&&HeadLight!=NONE) {
			CameraZoom = USARRemoteBot(Controller).myCameraZoom;
			if (CameraZoom==0) CameraZoom = degCameraDefFov; // reset
			else if (CameraZoom<=degCameraMinFov) CameraZoom = degCameraMinFov; // low bound
			else if (CameraZoom>degCameraMaxFov) CameraZoom = degCameraMaxFov; // upper bound
			USARRemoteBot(Controller).myCameraZoom=-1;
		}

	}

	if(Level.NetMode != NM_DedicatedServer)
	{
		// Set headlight projector state.
		if (Headlight.Rotation!=HLRotation)
			Headlight.SetRelativeRotation(HLRotation+default.HLRotation);
		Headlight.DetachProjector();
		if(HeadlightOn)
			Headlight.AttachProjector();

		// Update dust kicked up by wheels.
		Dust[0].UpdateDust(frontLeft, DustSlipRate, DustSlipThresh);
		Dust[1].UpdateDust(frontRight, DustSlipRate, DustSlipThresh);
	}

	// This assume the sound is an idle-ing sound, and increases with pitch
	// as wheels speed up.
	EnginePitch = 64 + ((WheelSpinSpeed/EnginePitchScale) * (255-64));
    SoundPitch = FClamp(EnginePitch, 0, 254);
	//Log("Engine Pitch:"$SoundPitch);
    //SoundVolume = 255;
}

function timer()
{
	local string outstring;
	local int i;

	outstring="STA {Time "$Level.TimeSeconds$"}"$
			     " {State "$myState$"}"$
			     " {Camera "$(HLRotation)$"}"$
			     " {Zoom "$CameraZoom$"}"$
			     " {CameraVel "$TiltSpeed$","$PanSpeed$",0.0}"$
			     " {Attitude "$Rotation$"}"$
			     " {Location "$Location$"}"$
			     " {LeftWheel "$frontLeft.Velocity$"}"$
			     " {RightWheel "$frontRight.Velocity$"}"$
			     " {Velocity "$Velocity$"}"$
			     " {LightToggle "$HeadlightOn$"}"$
			     " {LightIntensity "$HeadlightItensity$"}"$
			     " {Battery "$(batteryLife-myLife)$"}";
	USARRemoteBot(Controller).myConnection.SendLine(outstring);

	outstring = "";
	for (i=0;i<Sensors.length;i++) {
		if (ProcessedSensors[i]==0) {
			if (SensorList[i].bUseGroup)
				outstring = getGroupData(SensorList[i].ItemType);
			else
				outstring = SensorList[i].GetData();
			if (outstring != "")
				USARRemoteBot(Controller).myConnection.SendLine(SensorList[i].GetHead()@outstring);
		}
		}
	
	for (i=0;i<Sensors.length;i++)
		ProcessedSensors[i] = 0;
}

function String getGroupData(String type) {
	local String outstring;
	local int i;

	outstring = "";
	for (i=0;i<Sensors.length;i++) {
		if (SensorList[i].ItemType == type) {
			outstring = outstring@SensorList[i].GetData();
			ProcessedSensors[i] = 1;
		}
	}
	return outstring;
}

simulated event PhysicsVolumeChange( PhysicsVolume NewVolume )
{
	Super.PhysicsVolumeChange(NewVolume);
	log("Volume:"$NewVolume.LocationName@"State:"$GetStateName());
	if ( NewVolume.LocationName=="Papers")
		GotoState('CrossPapers','Begin');
	//if ( NewVolume.LocationName=="Stair")
	//	GotoState('ClimbStair','Begin');
}

state CrossPapers{

	function BeginState()
	{
		log("Enter state: CrossPapers");
	}

	function damp()
	{
		if (FRand()>0.3) {
    		frontLeft.RollFriction = TireRollFriction*0.5;
    		frontLeft.LateralFriction = TireLateralFriction*0.0;
    	}
    	else {
			frontLeft.RollFriction = TireRollFriction;
    		frontLeft.LateralFriction = TireLateralFriction;
    	}

		if (FRand()>0.3) {
			frontRight.RollFriction = TireRollFriction*0.5;
    		frontRight.LateralFriction = TireLateralFriction*0.0;
    	}
    	else {
			frontRight.RollFriction = TireRollFriction*FRand();
    		frontRight.LateralFriction = TireLateralFriction*FRand();
    	}

		frontLeft.WheelJoint.KUpdateConstraintParams();
		frontRight.WheelJoint.KUpdateConstraintParams();
	}

	function EndState()
	{
		log("Leave state: CrossPapers");
		KVehicleUpdateParams();
	}

Begin:
	damp();
	Sleep(0.2);
	if ( PhysicsVolume.LocationName=="Papers") goto('Begin');
	GotoState('USARCar');
}
/*
state ClimbStair{

	function BeginState()
	{
		log("Enter state: ClimbStair");
	}

	function EndState()
	{
		log("Leave state: ClimbStair");
		KVehicleUpdateParams();
	}

	function climb()
	{
		if (frontLeft.RollFriction>TireRollFriction) return;
	    frontLeft.RollFriction = TireRollFriction*100;
	    frontLeft.LateralFriction = TireRollFriction*150;
	    frontRight.RollFriction = TireRollFriction*100;
	    frontRight.LateralFriction = TireRollFriction*150;
		frontLeft.WheelJoint.KUpdateConstraintParams();
		frontRight.WheelJoint.KUpdateConstraintParams();
	}

Begin:
	climb();
	Sleep(0.2);
	if ( PhysicsVolume.LocationName=="Stair") goto('Begin');
	GotoState('USARCar');
}
*/
defaultproperties
{
     Kp = 0.2
     Ki = 0.7
     Kd = 0.0
     MinOut = -80.0
     MaxOut = 80.0

	 CameraDefFov=0.7854
	 CameraMinFov=0.3491
	 CameraMaxFov=2.0944

	 ConverterClass="USARBot.USARConverter"

     bDebug=true
     HLRotation=(Pitch=-2000,Yaw=0)
     HeadlightOffset=(Z=10.500000)
     HeadlightOn=True
     HeadlightItensity=100
     DustDrop=30.000000
     DustSlipRate=2.800000
     DustSlipThresh=0.100000
     TargetMaterial=FinalBlend'InterfaceContent.HUD.fbBombFocus'
     EnginePitchScale=655350.000000
     HitSoundThreshold=30.000000
     msgTimer=0.200000
     FrontTireClass=Class'USARBot.BcTire'
     TireDrawScale3D=(X=1.0,Y=0.3,Z=1.0)
     WheelFrontAlong=0.000000
     WheelFrontAcross=11.550
     WheelVert=-1.000000
     MaxBrakeTorque=100.000000
     SuspStiffness=150.000000
     SuspDamping=14.500000
     SuspHighLimit=0.500000
     SuspLowLimit=-0.500000
     TireRollFriction=5.000000
     TireLateralFriction=2.500000
     TireRollSlip=0.060000
     TireLateralSlip=0.040000
     TireSlipRate=0.007000
     TireSoftness=0.000000
     ChassisMass=1.000000
     TorqueCurve=(Points=((OutVal=10.000000),(InVal=150000.000000,OutVal=10.000000),(InVal=200000.000000,OutVal=6.000000),(InVal=300000.000000)))
     bSpecialHUD=True
     AirSpeed=5000.000000
	 AmbientGlow=48
     Health=800
	 batteryLife=3600
     DrawType=DT_StaticMesh
     StaticMesh=StaticMesh'USAR_Robots.Robots.Corky'
     DrawScale=0.300000
     SoundRadius=255.000000
     Begin Object Class=KarmaParamsRBFull Name=KParams0
         KInertiaTensor(0)=20.000000
         KInertiaTensor(3)=30.000000
         KInertiaTensor(5)=48.000000
         KCOMOffset=(X=-0.02,Z=-0.03)
         KLinearDamping=0.000000
         KAngularDamping=0.000000
         KStartEnabled=True
         bHighDetailOnly=False
         bClientOnly=False
         bKDoubleTickRate=True
         KFriction=0.0000000
         Name="KParams0"
     End Object
     KParams=KarmaParamsRBFull'USARBot.USARBc.KParams0'
}
