class USARBotConnection extends BotConnection;

var USARReplicationInfo gInfo;
var USARRemoteBot urBot;
var USARConverter converter;

function PostBeginPlay()
{
	super.PostBeginPlay();
	gInfo = USARReplicationInfo(Level.Game.GameReplicationInfo);
}

//InitRecieved from client
event InitRecieved()
{
	local string clientName;
	local string teamString;
	local string skinString;
	local int teamNum;
	local vector startLocation;
	local vector startRotation;
	local vector newLocation;
	local rotator newRotation;
	local string className;
	local class<Pawn> pawnClass;

    clientName = GetArgVal("Name");
    teamString = GetArgVal("Team");
    skinString = GetArgVal("Skin");
    ParseVector(startLocation,"Location");
    ParseRotAsVec(startRotation);
    className = GetArgVal("ClassName");

    if (teamString == "")
        teamNum = 255;
    else
        teamNum = int(teamString);

	if(bDebug)
		log("InitRecieved");

	if (className!="") {
		pawnClass = class<Pawn>(DynamicLoadObject(className, class'Class'));
		if (ClassIsChildOf(pawnClass,class'KRobot'))
			converter = new class<USARConverter>(DynamicLoadObject(class<KRobot>(pawnClass).default.ConverterClass, class'Class'));
		else if (ClassIsChildOf(pawnClass,class'USARCar'))
			converter = new class<USARConverter>(DynamicLoadObject(class<USARCar>(pawnClass).default.ConverterClass, class'Class'));
		else if (ClassIsChildOf(pawnClass,class'USARBc'))
			converter = new class<USARConverter>(DynamicLoadObject(class<USARBc>(pawnClass).default.ConverterClass, class'Class'));
	}
	if (converter!=None) {
		newLocation = converter.LengthVectorToUU(startLocation);
		newRotation = converter.RotatorToUU(startRotation);
	} else {
	newLocation = startLocation;
    newRotation.Roll  = startRotation.X;
    newRotation.Pitch = startRotation.Y;
    newRotation.Yaw   = startRotation.Z;
	}

	theBot = BotDeathMatch(Level.Game).AddRemoteBot(self, clientName, teamNum, newLocation, newRotation, className);
	urBot = USARRemoteBot(theBot);

	urBot.RobotName = theBot.PlayerReplicationInfo.PlayerName;
    urBot.SetSkin(skinString);

	if(bDebug)
    	if(theBot != None) log("added bot, now running");
    	else log("ERROR ADDING BOT");


    gotoState('monitoring','Running');
}

function ProcessAction(string cmdType)
{
    local rotator r;
    local vector v;
    local float distance;
    local byte argCount;    //Used in MULTIDRIVE command parsing

	super.ProcessAction(cmdType);

   	switch(cmdType)
   	{
	case "CAMERA":
	    if (GetArgVal("Rotation")!="") {
			ParseRotAsVec(v);
			if (converter!=None) {
				urBot.myCameraRotation = converter.RotatorToUU(v);
			} else {
			urBot.myCameraRotation.Pitch=v.Y;
			urBot.myCameraRotation.Yaw=v.Z;
			urBot.myCameraRotation.Roll=v.X;
		}
		}
		urBot.myCameraRotationOrder=0; // default 0
	    if (GetArgVal("Zoom")!="")
			urBot.myCameraZoom=converter.AngleToDeg(float(GetArgVal("Zoom")));
	    if (GetArgVal("Order")!="")
			urBot.myCameraRotationOrder=byte(int(GetArgVal("Order")));
	    break;
	case "TURN": // old command. Ignore it!
	    if (GetArgVal("Rotation")!="") {
			ParseRotAsVec(v);
			if (converter!=None) {
				r = converter.RotatorToUU(v);
			} else {
				r.Pitch=v.Y;
				r.Yaw=v.Z;
				r.Roll=v.X;
	    	}
			urBot.myTurn.Pitch=r.Pitch;
			urBot.myTurn.Yaw=urBot.Rotation.Yaw+r.Yaw;
			urBot.myTurn.Roll=r.Roll;
	    }
	    distance = float(GetArgVal("Distance"));
	    if(distance != 0.0) {
	    	if (converter!=None)
	    		urBot.myDistance = converter.LengthToUU(distance);
	    	else
		    	urBot.myDistance = distance;
		}
		break;
	case "DRIVE":
		// two wheel vehicle
		urBot.bNewThrottle = false;
	    if (GetArgVal("Left")!="") {
			urBot.LeftThrottle = float(GetArgVal("Left"));
			urBot.bNewThrottle = true;
		}
	    if (GetArgVal("Right")!="") {
			urBot.RightThrottle = float(GetArgVal("Right"));
			urBot.bNewThrottle = true;
		}
		
        if (GetArgVal("Speed")!="")
		{
		    urBot.Speed = float(GetArgVal("Speed"));
		    urBot.bNewThrottle = true;
		}
                
		if (GetArgVal("FrontSteer")!="")
		{
		    urBot.FrontSteer = float(GetArgVal("FrontSteer"));
		    urBot.bNewThrottle = true;
		}
                
		if(GetArgVal("RearSteer")!="")
		{
		    urBot.RearSteer = float(GetArgVal("RearSteer"));
		    urBot.bNewThrottle = true;
		}

		if (GetArgVal("Normalized")!="")
			urBot.Normalized = bool(GetArgVal("Normalized"));
		else
			urBot.Normalized = false;
		// head light on'off
	    if (GetArgVal("Light")!="")
			urBot.Light = bool(GetArgVal("Light"));
		// flip the vehicle
	    if (GetArgVal("Flip")!="")
			urBot.Flip = bool(GetArgVal("Flip"));
		// general joint control
	    if (GetArgVal("Steer")!="" && GetArgVal("Steer")!="None" && GetArgVal("Steer")!="none") {
	    	if (converter!=None)
				urBot.JSteer[urBot.JointControlIdx] = converter.AngleToUU(float(GetArgVal("Steer")));
			else
			urBot.JSteer[urBot.JointControlIdx] = float(GetArgVal("Steer"));
		} else
			urBot.JSteer[urBot.JointControlIdx] = 65535; // 65535 means do not steer
	    if (GetArgVal("Order")!="")
			urBot.JOrder[urBot.JointControlIdx] = byte(int(GetArgVal("Order")));
	    if (GetArgVal("Value")!="") {
			if (urBot.JOrder[urBot.JointControlIdx]==0 ||
			    urBot.JOrder[urBot.JointControlIdx]==1)
			    urBot.JValue[urBot.JointControlIdx] = converter.AngleToUU(float(GetArgVal("Value")));
			else
			urBot.JValue[urBot.JointControlIdx] = float(GetArgVal("Value"));
		}
	    if (GetArgVal("Name")!="") {
			urBot.JName[urBot.JointControlIdx] = GetArgVal("Name");
            urBot.JointControlIdx+=1;
		}
	    break;

    // The following command is designed to operate multiple joints at once
    // with order 0 (angle position).
    // This allows to mantain movement synchronization of complex robots.
    case "MULTIDRIVE":
         for(argCount=0; argCount < attrNum; argCount++)
         {
            //We store all commands.
            //ProcessCarInput function will discard invalid joints.
            urBot.JName[urBot.JointControlIdx] = ReceivedArgs[argCount];
            urBot.JOrder[urBot.JointControlIdx] = 0;
            urBot.JValue[urBot.JointControlIdx] = converter.AngleToUU(float(ReceivedVals[argCount]));
            urBot.JointControlIdx++;
         }
        break;

	case "SENSOR":
		urBot.SetName = GetArgVal("Name");
	    if (GetArgVal("Scan")!="")
			urBot.RangeScan = bool(GetArgVal("Scan"));
		break;
	case "GETGEO":
		urBot.GeoType = GetArgVal("Type");
		urBot.GeoName = GetArgVal("Name");
		break;
	case "GETCONF":
		urBot.ConfType = GetArgVal("Type");
		urBot.ConfName = GetArgVal("Name");
		break;
	case "SET":
		if (GetArgVal("Client")!="") {
			gInfo.cltRobot = GetArgVal("Robot");
			gInfo.cltCamera = GetArgVal("Name");
			gInfo.cltIP = GetArgVal("Client");
			break;
		}

		urBot.SetType = GetArgVal("Type");
		urBot.SetName = GetArgVal("Name");
		urBot.Opcode = GetArgVal("Opcode");
		urBot.Params = GetArgVal("Params");

		if (urBot.SetType=="RangeScanner") {
			if (GetArgVal("Scan")!="")
				urBot.RangeScan = bool(GetArgVal("Scan"));
		} else if (urBot.SetType=="IRCamera") {
			if (GetArgVal("Scan")!="")
				//urBot.IRCameraScan = true;
				urBot.IRCameraScan = bool(GetArgVal("Scan"));
		} else if (urBot.SetType=="INU") {
			if (GetArgVal("Reset")!="")
				urBot.SetOrientation = bool(GetArgVal("Reset"));
		} else if (urBot.SetType=="Odometry") {
			if (GetArgVal("Reset")!="")
				urBot.SetPosition = bool(GetArgVal("Reset"));
		} else if (urBot.SetType=="Encoder") {
			if (GetArgVal("Reset")!="")
				urBot.SetEncoder = bool(GetArgVal("Reset"));
		}
		break;
	case "MISPKG":
		urBot.MPName = GetArgVal("Name");
	    if (GetArgVal("Rotation")!="") {
			ParseRotAsVec(v);
			if (converter!=None) {
				urBot.MPRotation = converter.RotatorToUU(v);
			} else {
			urBot.MPRotation.Pitch=v.Y;
			urBot.MPRotation.Yaw=v.Z;
			urBot.MPRotation.Roll=v.X;
		}
		}
	    if (GetArgVal("Translation")!="") {
			ParseVector(v,"Translation");
			if (converter!=None)
				urBot.MPTranslation = converter.LengthVectorToUU(v);
			else
			urBot.MPTranslation=v;
		}
		urBot.MPOrder=0; // default 0
	    if (GetArgVal("Order")!="")
			urBot.MPOrder=byte(int(GetArgVal("Order")));
		break;
	}
}
