class USARRobot extends Pawn config(ExpControl);

var const vector	HeadlightOffset;
var USARHeadlight	Headlight;
var rotator HLRotation;
var bool HeadlightOn;
var float CameraZoom;

var bool bExp;
var bool bInitExp;
var int tExp;
var config int ExpTimer;
var config float expTime;
var config string expIntro;
var float startTime;
var float utTime;
var int expX, expY;
var int expX1, expY1;
var int expX2, expY2;
var ExpPath ePath;
var texture ExpTarget;
var config float ExpTargetScale;
var vector vTarget;
var rotator rTarget;

var float msgTimer;

replication {
	reliable if(Role == ROLE_Authority) HLRotation, CameraZoom, expX, expY, utTime;
}


simulated function PostBeginPlay()
{
	local vector RotX, RotY, RotZ;

	super.PostBeginPlay();
	// Create headlight projector. We make sure it doesn't project on the vehicle itself.
	GetAxes(Rotation,RotX,RotY,RotZ);
	Headlight = spawn(class'USARHeadlight', self,, Location + HeadlightOffset.X * RotX + HeadlightOffset.Y * RotY + HeadlightOffset.Z * RotZ);
	if (Headlight!=None) {
		Headlight.SetBase(self);
		Headlight.SetRelativeRotation(rot(0, 0, 0));
	}
	if (bExp) ePath=new class'USARBot.ExpPath';
	CameraZoom=1.0;
	SetTimer(msgTimer,true);
}

simulated event Destroyed()
{
	Headlight.Destroy();
	super.Destroyed();
}

simulated function Tick( float DeltaTime )
{
	local vector RotX, RotY, RotZ;

	if (Headlight == None) Return;

	if (Role==ROLE_Authority)
		HLRotation +=USARRemoteBot(Controller).myCameraRotation;
	Headlight.SetRelativeRotation(HLRotation);

	Headlight.DetachProjector();
	if(HeadlightOn)
		Headlight.AttachProjector();

	if (bExp) {
		if ((startTime==0)&&(USARRemoteBot(Controller).myCameraRotation.Pitch!=0||USARRemoteBot(Controller).myCameraRotation.Yaw!=0||USARRemoteBot(Controller).botDR!=none)) startTime=Level.TimeSeconds;
		if (startTime>0) utTime=Level.TimeSeconds-startTime;
		if (!bInitExp) {
			//ePath.setOffset((C.SizeX-ExpTarget.MaterialUSize()*ExpTargetScale)/2,(C.SizeY-ExpTarget.MaterialVSize()*ExpTargetScale)/2);
			//ePath.setScale(C.SizeX,C.SizeY);
			ePath.setOffset(0,-50);
			ePath.setScale(100,100);
			//ePath.dump();
			bInitExp=true;
			ePath.nextPoint(expX1,expY1);
			ePath.nextPoint(expX2,expY2);
		}
		if (tExp>ExpTimer) {
			expX1=expX2;
			expY1=expY2;
			ePath.nextPoint(expX2,expY2);
			tExp=0;
		}
		expX=expX1+(expX2-expX1)*tExp/ExpTimer;
		expY=expY1+(expY2-expY1)*tExp/ExpTimer;
		GetAxes(Rotation,RotX,RotY,RotZ);
		vTarget=Headlight.Location + 400*RotX+expX*RotY+expY*RotZ;
		rTarget=rotator(400*RotX+expX*RotY+expY*RotZ)-Rotation;
		tExp++;
	}
	//log("X="$expX$" Y="$expY$" tExp="$tExp);
	//log("DR_V:"$(USARRemoteBot(Controller)).botDR.current.DR_V@(USARRemoteBot(Controller)).botDR.cache.DR_V);
}

// For drawing targetting reticule
simulated function DrawHud(Canvas C)
{
	local vector ScreenPos;

	if (bExp) {
		C.Style = ERenderStyle.STY_Masked;
		ScreenPos = C.WorldToScreen( vTarget);
		C.SetPos(ScreenPos.X, ScreenPos.Y);
		C.DrawIcon(ExpTarget,ExpTargetScale);
		C.Style = ERenderStyle.STY_Normal;
		C.DrawColor.R = 255;
		C.DrawColor.G = 255;
		C.DrawColor.B = 255;
		C.DrawColor.A = 255;
		C.FontScaleX=3;
		C.FontScaleY=3;
		C.SetPos(C.SizeX-320,0);
		C.DrawText("LeftTime:"$(expTime-utTime));
		C.FontScaleX=2;
		C.FontScaleY=2;
		C.SetPos(C.SizeX-50,7);
		C.DrawText("sec");
		if (utTime<15) DrawMultiLine(C,0.1,0.05,0.9,0.8,expIntro);
		if (utTime>expTime) {
			C.FontScaleX=3;
			C.FontScaleY=3;
			C.DrawColor.G = 0;
			C.DrawColor.B = 0;
			C.SetPos(150,250);
			C.DrawText("Time Out! Please Alert Your Experimentor.");
		}
	}
}

simulated function DrawMultiLine(Canvas Canvas, float top, float left, float width, float height, string text)
{
local string TextCopy, Line;
local int ltp;
local float XL, YL, xTop, xLeft;

	Canvas.TextSize(text, XL, YL);
	ltp = (height * Canvas.SizeY) / YL; // Max lines
	xTop = top * Canvas.SizeY;
	xLeft = left * Canvas.SizeX;
	TextCopy = text;
	Canvas.WrapText(TextCopy, Line, width * Canvas.SizeX, Canvas.Font, Canvas.FontScaleX);
	while (ltp > 0)
	{
		if (Len(Line) > 0) {
			Canvas.SetPos(xLeft,xTop);
			Canvas.DrawText(Line);
		}

		xTop += YL;
		Canvas.WrapText(TextCopy, Line, width * Canvas.SizeX, Canvas.Font, Canvas.FontScaleX);
		if (TextCopy == "" && Line == "")
			break;
	}
}


function timer()
{
	local string outstring;

	outstring="STA {Time "$Level.TimeSeconds$"}"$
				 " {Location "$Location$"}"$
				 " {Rotation "$Rotation$"}"$
				 " {Velocity "$Velocity$"}"$
				 " {Camera "$HLRotation$"}"$
				 " {Target "$rTarget$"}"$
			     " {Trace "$(HLRotation-rTarget)$"}";
	USARRemoteBot(Controller).myConnection.SendLine(outstring);
}

defaultproperties
{
     bExp=true
     bInitExp=false
     ExpTarget=Texture'Engine.S_LookTarget'
     ExpTargetScale=1.0
     ExpTimer=10
     msgTimer=0.1
	 bSpecialHUD=True
     HeadlightOffset=(Z=100.000000)
     HeadlightOn=false
     Physics=PHYS_Falling
     // FMP
     // Mesh=VertMesh'USARBot.t80'
     DrawScale3D=(X=3.000000,Y=4.000000,Z=5.000000)
     // FMP
     // Skins(0)=Texture'USARBot.Skins.Jtex1'
     Skins(1)=Texture'PlayerSkins.JuggMaleAHeadA'
     CollisionRadius=40.000000
     CollisionHeight=8.000000
}

